import sys
sys.path.append("..")

import json
from resources.hasher import APIHash
import requests

headers = {"Accept": "application/json", "Content-Type": "application/x-www-form-urlencoded"}

def verifyPayment (creds, txnid):
    payload = {
        "key": creds["key"],
        "command": "verify_payment",
        "var1": txnid,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text


def checkPayment (creds, payuid):
    payload = {
        "key": creds["key"],
        "command": "check_payment",
        "var1": payuid,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text

def cancelRefundTransaction (creds, payuid, tokenid, amount):
    payload = {
        "key": creds["key"],
        "command": "cancel_refund_transaction",
        "var1": payuid,
        "var2": tokenid,
        "amount": amount,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text

def checkIsDomestic (creds, bin):
    payload = {
        "key": creds["key"],
        "command": "check_isDomestic",
        "var1": bin,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text

def eligibleBinsForEMI(creds, bin):
    payload = {
        "key": creds["key"],
        "command": "eligibleBinsForEMI",
        "var1": "bin",
        "var2": bin,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text



def getEmiAmountAccordingToInterest(creds, amount):
    payload = {
        "key": creds["key"],
        "command": "getEmiAmountAccordingToInterest",
        "var1": amount,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text

def getCheckoutDetails(creds, params):
    payload = {
        "key": creds["key"],
        "command": "get_checkout_details",
        "var1": json.dumps(params),
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text

def getSettlementDetails(creds, dateOrUTR):
    payload = {
        "key": creds["key"],
        "command": "get_settlement_details",
        "var1": dateOrUTR,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text

def getNetbankingStatus(creds, bankcode):
    payload = {
        "key": creds["key"],
        "command": "getNetbankingStatus",
        "var1": bankcode,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text

def getIssuingBankStatus(creds, bin):
    payload = {
        "key": creds["key"],
        "command": "getIssuingBankStatus",
        "var1": bin,
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text

def createInvoice(creds, params):
    payload = {
        "key": creds["key"],
        "command": "create_invoice",
        "var1": json.dumps(params),
        "hash": ""
    }
    payload["hash"] = APIHash(creds, payload)
    response = requests.request("POST", creds["apiURL"], data=payload, headers=headers)
    return response.text


class Base(object):
    _instance = None
    key = ""
    salt = ""
    paymentURL = ""
    apiURL = ""

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Base, cls).__new__(
                                cls, *args, **kwargs)
        return cls._instance

    def set_params(self,params):
        self.key = params.get("key")
        self.salt = params.get("salt")
        self.env = params.get("env")
        self.paymentURL = params.get("paymentURL")
        self.apiURL = params.get("apiURL")
        return (self.key ,self.salt, self.paymentURL, self.apiURL) 

    def get_params(self):
        return ({
            "key": self.key,
            "salt":self.salt,
            "paymentURL":self.paymentURL,
            "apiURL": self.apiURL
        })





class ERROR_CODE(object):
    BAD_REQUEST_ERROR_CODE = "BAD_REQUEST_ERRORCODE"
    GATEWAY_ERROR_CODE = "GATEWAY_ERROR_CODE"
    SERVER_ERROR_CODE = "SERVER_ERROR_CODE"
    API_REQUEST_ERROR_CODE = "API_REQUEST_ERROR_CODE"


import sys
sys.path.append("..")

from resources.hasher import generate_hash
from typing import Dict

def generate_payment_form(creds: Dict[str, str], params: Dict[str, str]) -> str:
    mandatory_params = ["txnid", "amount", "productinfo", "firstname", "email", "phone", "surl", "furl"]
    for param_name in mandatory_params:
        if param_name not in params:
            raise ValueError(f"Missing mandatory parameter '{param_name}'")
    params["hash"] = generate_hash(creds, params)
    params["key"] = creds["key"]
    form = "<form name=\"payment_post\" id=\"payment_post\" action=\"" + creds["paymentURL"] + "\" method=\"post\">"
    for key, value in params.items():
        form += f"<input hidden type=\"text\" name=\"{key}\" value=\"{value}\"/>"
    form += "</form><script type=\"text/javascript\"> window.onload=function(){document.forms['payment_post'].submit();}</script>"
    return form
        

         
        












from setuptools import setup, find_packages

with open('README.md') as readme:
    readme_content = readme.read()

setup(
    name='payu-websdk',
    version='0.1.2',
    description='PayU',
    author='PayU Web SDK for Python',
    author_email='integration@payu.in',
    url='https://github.com/payu-india/web-sdk-python',
    packages=find_packages(),
    long_description=readme_content,
    long_description_content_type='text/markdown',
    license="MIT",
    install_requires=[
    ],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License'
    ],
)


import hashlib

def generate_hash(credes,params):
    key = credes.get('key')
    salt = credes.get('salt')
    txnid = params.get("txnid")
    amount = params.get("amount")
    productinfo = params.get("productinfo", "")
    firstname = params.get("firstname", "")
    email = params.get("email", "")
    udf1 = params.get("udf1", "")
    udf2 = params.get("udf2", "")
    udf3 = params.get("udf3", "")
    udf4 = params.get("udf4", "")
    udf5 = params.get("udf5", "")
    if params.get("additional_charges") is not None and params.get("additional_charges") != "":
        additional_charges = params.get("additional_charges")
        payment_hash_sequence = f"{key}|{txnid}|{amount}|{productinfo}|{firstname}|{email}|{udf1}|{udf2}|{udf3}|{udf4}|{udf5}||||||{salt}|{additional_charges}"
    else:
        payment_hash_sequence = f"{key}|{txnid}|{amount}|{productinfo}|{firstname}|{email}|{udf1}|{udf2}|{udf3}|{udf4}|{udf5}||||||{salt}"
    hash_value = hashlib.sha512((payment_hash_sequence).encode('utf-8')).hexdigest()
    return hash_value

def generate_reverse_hash(credes, params):
    key = credes['key']
    salt = credes['salt']
    txnid = params.get("txnid")
    amount = params.get("amount")
    productinfo = params.get("productinfo")
    firstname = params.get("firstname")
    email = params.get("email")
    udf1 = params.get("udf1", "")
    udf2 = params.get("udf2", "")
    udf3 = params.get("udf3", "")
    udf4 = params.get("udf4", "")
    udf5 = params.get("udf5", "")
    status = params.get("status")
    if params.get("additionalCharges") is not None and params.get("additionalCharges") != "":
        additional_charges = params.get("additionalCharges")
        validate_hash_sequence = f"{additional_charges}|{salt}|{status}||||||{udf5}|{udf4}|{udf3}|{udf2}|{udf1}|{email}|{firstname}|{productinfo}|{amount}|{txnid}|{key}"
    else:
        validate_hash_sequence = f"{salt}|{status}||||||{udf5}|{udf4}|{udf3}|{udf2}|{udf1}|{email}|{firstname}|{productinfo}|{amount}|{txnid}|{key}"
    validate_hash = hashlib.sha512((validate_hash_sequence).encode('utf-8')).hexdigest().lower()
    return validate_hash

def validate_hash(creds, params) -> bool:
    if (generate_reverse_hash(creds,params) == params["hash"]):
        return True
    return False

def APIHash(credes, params):
    key = credes['key']
    salt = credes['salt']
    command = params.get("command")
    var1 = params.get("var1")
    hash_sequence = f"{key}|{command}|{var1}|{salt}"
    hash_value = hashlib.sha512((hash_sequence).encode('utf-8')).hexdigest().lower()
    return hash_value

